@@ -1,10 +1,12 @@
 mod blockchain_api;
 mod builtin_function_names;
 mod call_value_api;
+mod composite_api;
 mod crypto_api;
 mod endpoint_arg_api;
 mod endpoint_finish_api;
 mod error_api;
+mod external_view;
 mod log_api;
 mod managed_types;
 mod print_api;
@@ -13,16 +15,18 @@ mod storage_api;
 pub mod uncallable;
 mod vm_api;
 
-pub use blockchain_api::BlockchainApi;
+pub use blockchain_api::*;
 pub use builtin_function_names::*;
-pub use call_value_api::CallValueApi;
-pub use crypto_api::CryptoApi;
-pub use endpoint_arg_api::EndpointArgumentApi;
-pub use endpoint_finish_api::EndpointFinishApi;
-pub use error_api::ErrorApi;
-pub use log_api::LogApi;
+pub use call_value_api::*;
+pub use composite_api::*;
+pub use crypto_api::*;
+pub use endpoint_arg_api::*;
+pub use endpoint_finish_api::*;
+pub use error_api::*;
+pub use external_view::*;
+pub use log_api::*;
 pub use managed_types::*;
-pub use print_api::PrintApi;
-pub use send_api::SendApi;
+pub use print_api::*;
+pub use send_api::*;
 pub use storage_api::*;
 pub use vm_api::VMApi;
