@@ -4,6 +4,14 @@ use alloc::vec::Vec;
 use super::Handle;
 
 pub trait StorageReadApi {
+    type StorageReadApiImpl: StorageReadApiImpl;
+
+    fn storage_read_api_impl() -> Self::StorageReadApiImpl;
+}
+
+pub trait StorageReadApiImpl {
+    fn storage_read_api_init(&self) {}
+
     fn storage_load_len(&self, key: &[u8]) -> usize;
 
     fn storage_load_vec_u8(&self, key: &[u8]) -> Vec<u8>;
@@ -21,9 +29,17 @@ pub trait StorageReadApi {
     fn storage_load_u64(&self, key: &[u8]) -> u64;
 
     fn storage_load_i64(&self, key: &[u8]) -> i64;
+
+    fn storage_load_from_address(&self, address_handle: Handle, key_handle: Handle) -> Handle;
 }
 
 pub trait StorageWriteApi {
+    type StorageWriteApiImpl: StorageWriteApiImpl;
+
+    fn storage_write_api_impl() -> Self::StorageWriteApiImpl;
+}
+
+pub trait StorageWriteApiImpl {
     fn storage_store_slice_u8(&self, key: &[u8], value: &[u8]);
 
     fn storage_store_big_uint_raw(&self, key: &[u8], value_handle: Handle);
