@@ -98,7 +98,7 @@ where
     M: ManagedTypeApi,
 {
     fn push_dyn_arg<O: DynArgOutput>(&self, output: &mut O) {
-        (&self).push_dyn_arg(output)
+        ContractCallArg::push_dyn_arg(&self, output)
     }
 }
 
