@@ -1,6 +1,8 @@
+use core::marker::PhantomData;
+
 use crate::{
     abi::{OutputAbi, TypeAbi, TypeDescriptionContainer},
-    api::SendApi,
+    api::{CallTypeApi, SendApiImpl},
     io::EndpointResult,
     types::{BigUint, ManagedAddress, ManagedBuffer},
 };
@@ -8,30 +10,43 @@ use alloc::{string::String, vec::Vec};
 
 pub struct SendEgld<SA>
 where
-    SA: SendApi + 'static,
+    SA: CallTypeApi + 'static,
 {
-    pub api: SA,
+    _phantom: PhantomData<SA>,
     pub to: ManagedAddress<SA>,
     pub amount: BigUint<SA>,
     pub data: ManagedBuffer<SA>,
 }
 
+impl<SA> SendEgld<SA>
+where
+    SA: CallTypeApi + 'static,
+{
+    pub fn new(to: ManagedAddress<SA>, amount: BigUint<SA>, data: ManagedBuffer<SA>) -> Self {
+        Self {
+            _phantom: PhantomData,
+            to,
+            amount,
+            data,
+        }
+    }
+}
+
 impl<SA> EndpointResult for SendEgld<SA>
 where
-    SA: SendApi + 'static,
+    SA: CallTypeApi + 'static,
 {
     type DecodeAs = ();
 
     #[inline]
-    fn finish<FA>(&self, _api: FA) {
-        self.api
-            .direct_egld(&self.to, &self.amount, self.data.clone());
+    fn finish<FA>(&self) {
+        SA::send_api_impl().direct_egld(&self.to, &self.amount, self.data.clone());
     }
 }
 
 impl<SA> TypeAbi for SendEgld<SA>
 where
-    SA: SendApi + 'static,
+    SA: CallTypeApi + 'static,
 {
     fn type_name() -> String {
         "SendEgld".into()
